import javax.sound.midi.*;


public class MidiWrapper
{
private ShortMessage voiceMessage = new ShortMessage();
private int midiChannel = 0;

/* Receiver receives commands but actually sends MIDI events to a
 * MIDI device
 */
private Receiver midireceiver = null;


  public MidiWrapper() // constructor
  {
    try
    {
      midireceiver = MidiSystem.getReceiver();
    }
    catch (MidiUnavailableException e)
    {
      System.out.println(e);
      System.exit(1);
    }

    if (midireceiver == null)
    {
      System.out.println("No midireceiver found");
      System.exit(1);
    }
  } // MidiWrapper()


  protected void finalize()
  {
    midireceiver.close();
  }

  public void end()
  {
    midireceiver.close();
    System.exit(1);
  }


  public void pause(int duration)
  {
    try
    {
      Thread.sleep(duration);
    }
    catch(InterruptedException e)
    {
      System.out.println(e);
    }
  } // pause()


  void setMidiChannel(int midiChannel)
  {
    this.midiChannel=midiChannel;
  }

  void programChange(int midiChannel,int midiProgram)
  {
    try{ // construct a new message
      voiceMessage.setMessage(ShortMessage.PROGRAM_CHANGE,
        midiChannel,midiProgram,0);
    }
    catch(InvalidMidiDataException e)
    {
      System.out.println(e);
      System.exit(1);
    }
  }


  /*
   * This version of playNote() does not expect a duration and only
   *  switches the note ON
   * To switch the note OFF, use endNote()
   */
  public void playNote(int noteNumber,int velocity)
  {
    // Prevent out-of-bounds exceptions
    if(noteNumber > 127) noteNumber = 127;
    if(noteNumber < 0) noteNumber = 0;
    if(velocity > 127) velocity = 127;
    if(velocity < 0) velocity = 0;

    try{ // construct a new message
      voiceMessage.setMessage(ShortMessage.NOTE_ON,
        midiChannel,noteNumber,velocity);
    }
    catch(InvalidMidiDataException e)
    {
      System.out.println(e);
      System.exit(1);
    }

    // Send the message
    midireceiver.send(voiceMessage, -1);
  } // playNote()


  /*
   * This version of playNote() expects a duration and switches
   *  the note ON and after a pause switches it OFF again
   */
  public void playNote(int noteNumber,int velocity,int duration)
  {
    // Prevent out-of-bounds exceptions
    if(noteNumber > 127) noteNumber = 127;
    if(noteNumber < 0) noteNumber = 0;
    if(velocity > 127) velocity = 127;
    if(velocity < 0) velocity = 0;

    try{ // construct a new message
      voiceMessage.setMessage(ShortMessage.NOTE_ON,
        midiChannel,noteNumber,velocity);
    }
    catch(InvalidMidiDataException e)
    {
      System.out.println(e);
      System.exit(1);
    }

    // Send the message
    midireceiver.send(voiceMessage, -1);

    pause(duration);

    try{ // construct a new message
      voiceMessage.setMessage(ShortMessage.NOTE_OFF,
        midiChannel,noteNumber,velocity);
    }
    catch(InvalidMidiDataException e)
    {
      System.out.println(e);
      System.exit(1);
    }

    midireceiver.send(voiceMessage, -1);
  } // playNote()


  public void endNote(int noteNumber)
  {
    // Prevent out-of-bounds exceptions
    if(noteNumber > 127) noteNumber = 127;
    if(noteNumber < 0) noteNumber = 0;

    try{ // construct a new message
      voiceMessage.setMessage(ShortMessage.NOTE_OFF,
        midiChannel,noteNumber,0);
    }
    catch(InvalidMidiDataException e)
    {
      System.out.println(e);
      System.exit(1);
    }

    midireceiver.send(voiceMessage, -1);
  } // endNote()

} // MidiWrapper{}
